/*
 * Copyright (c) 2004, 2005 Nokia
 * Authors: Lauri Arimo <lauri.arimo@movial.fi>
 *          Timo Savola <tsavola@movial.fi>
 *
 * sb_perl_wrapper is licensed under GPL.
 * You can read the full license at http://www.gnu.org/licenses/gpl.txt
 */

#ifndef _GNU_SOURCE
# define _GNU_SOURCE
#endif

#include "sb_config.hh"
#include <iostream>
#include <iterator>
#include <cstring>
#include <cerrno>
#include <unistd.h>

using namespace std;

#define HOST_LIBDIR    "/host_usr/perl/lib"

#define DEVKIT_ROOT    "/scratchbox/devkits/"
#define DEVKIT_LIBDIR  "/lib/perl"
#define DEVKIT_LIBDIR5 "/lib/perl5"

#define PERL_BIN       "/scratchbox/tools/bin/perl.bin"

static void error(const string &s)
{
	cerr << "sb_perl_wrapper: " << s << endl;
	exit(1);
}

static void error()
{
	error(strerror(errno));
}

int main(int old_argc, char **old_argv)
{
	try {
		// Modify environment

		string libpath = HOST_LIBDIR;

		if (char *oldpath = getenv("LD_LIBRARY_PATH")) {
			libpath += ":";
			libpath += oldpath;
		}

		if (setenv("LD_LIBRARY_PATH", libpath.c_str(), 1) < 0)
			error();

		// Modify arguments

		list<string> args;
		for (int i = 1; i < old_argc; ++i)
			args.push_back(old_argv[i]);

		map<string,string> config;
		sb::read_config(config);

		string names = config["SBOX_DEVKIT_NAMES"];
		string::size_type index = 0;

		insert_iterator<list<string> > inserter(args, args.begin());

		while (true) {
			string::size_type start = index;
			index = names.find(':', start);

			string name = names.substr(start, index - start);
			if (!name.empty()) {
				string opt;

				opt = "-I";
				opt += DEVKIT_ROOT;
				opt += name;
				opt += DEVKIT_LIBDIR;
				*inserter = opt;

				opt = "-I";
				opt += DEVKIT_ROOT;
				opt += name;
				opt += DEVKIT_LIBDIR5;
				*inserter = opt;
			}

			if (index == string::npos)
				break;

			++index;
		}

		char **new_argv = sb::build_argv(old_argv[0], args);

		// Execute

		execv(PERL_BIN, new_argv);
		cerr << PERL_BIN << ": " << strerror(errno) << endl;

	} catch (const exception &e) {
		error(e.what());
	}

	return 1;
}
